//+------------------------------------------------------------------+
//|                                       Bollinger Bands System.mq5 |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#include <MovingAverages.mqh>

#property description "An indicator to test strateg"
#property description "Draws arrows set by Unicode characters, on a chart"
#property description "The color, size, shift and symbol code of the arrow are changed in a random way"
#property description "after every N ticks"
#property description "The code parameter sets the base value: code=159 (a circle)"

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_plots   4
//--- basis
#property indicator_label1  "Basis"
#property indicator_type1   DRAW_LINE
#property indicator_color1  Black
//--- upper band
#property indicator_label2  "Upper Band"
#property indicator_type2   DRAW_LINE
#property indicator_color2  Black
//--- lower band
#property indicator_label3  "Lower Band"
#property indicator_type3   DRAW_LINE
#property indicator_color3  Black
//--- arrows
#property indicator_label4  "Arrows"
#property indicator_type4   DRAW_ARROW
#property indicator_color4  Red
#property indicator_width4  3
//--- input parametrs
input int     InpBandsPeriod=20;       // Period
input double  InpBandsDeviations=2.0;  // Deviation
input int     InpExpiration=1;         // Expiration
//--- global variables
int           ExtPlotBegin=0;
//---- indicator buffer
double        ExtBasisBuffer[];
double        ExtUpperBuffer[];
double        ExtLowerBuffer[];
double        ExtStdDevBuffer[];
double        ExtWinBuffer[];
double        ExtLossBuffer[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
//--- define buffers
   SetIndexBuffer(0,ExtBasisBuffer);
   SetIndexBuffer(1,ExtUpperBuffer);
   SetIndexBuffer(2,ExtLowerBuffer);
   SetIndexBuffer(3,ExtStdDevBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,ExtWinBuffer,INDICATOR_DATA);
   SetIndexBuffer(5,ExtLossBuffer,INDICATOR_DATA);
//--- set index labels
   PlotIndexSetString(0,PLOT_LABEL,"Basis ("+string(InpBandsPeriod)+", "+string(InpBandsDeviations)+")");
   PlotIndexSetString(1,PLOT_LABEL,"Upper Band ("+string(InpBandsPeriod)+", "+string(InpBandsDeviations)+")");
   PlotIndexSetString(2,PLOT_LABEL,"Lower Band ("+string(InpBandsPeriod)+", "+string(InpBandsDeviations)+")");
//--- indicator name
   IndicatorSetString(INDICATOR_SHORTNAME,"Bollinger Bands System");
//--- indexes draw begin settings
   ExtPlotBegin=InpBandsPeriod-1;
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,InpBandsPeriod);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,InpBandsPeriod);
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,InpBandsPeriod);
   PlotIndexSetInteger(4,PLOT_ARROW,108);
   //PlotIndexSetInteger(4,PLOT_ARROW_SHIFT,5);
   PlotIndexSetInteger(5,PLOT_ARROW,108);
   //PlotIndexSetInteger(5,PLOT_ARROW_SHIFT,5);
//--- Set as an empty value 0
   PlotIndexSetDouble(4,PLOT_EMPTY_VALUE,0);
   PlotIndexSetDouble(5,PLOT_EMPTY_VALUE,0);
//--- number of digits of indicator value
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
//---- OnInit done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//--- variables
   int i,limit;
   int e=InpExpiration;
   double wins=0;
   double losses=0;
//--- preliminary calculations
   if(prev_calculated==0)
     {
      //--- set first candle
      limit=1;
     }
   else limit=prev_calculated-1;
//--- the main loop of calculations
   for(i=limit;i<rates_total && !IsStopped();i++)
     {
      //--- bollinger bands
      ExtBasisBuffer[i]=SimpleMA(i,InpBandsPeriod,close);                           //--- basis
      ExtStdDevBuffer[i]=StdDev_Func(i,close,ExtBasisBuffer,InpBandsPeriod);        //--- StdDev
      ExtUpperBuffer[i]=ExtBasisBuffer[i]+InpBandsDeviations*ExtStdDevBuffer[i];    //--- upper band
      ExtLowerBuffer[i]=ExtBasisBuffer[i]-InpBandsDeviations*ExtStdDevBuffer[i];    //--- lower band
      //--- signals
      int call=close[i]<ExtLowerBuffer[i];      //--- call signal
      int put=close[i]>ExtUpperBuffer[i];       //--- put signal
      //--- results for calls
      if(call>0)
         {
          if(close[i]<close[i-e])
            {
             ExtWinBuffer[i]=low[i]; wins++;
            } else
               {
                ExtLossBuffer[i]=low[i]; losses++;
               }
         }
      //--- results for puts
      if(put>0)
         {
          if(close[i]>close[i-e])
            {
             ExtWinBuffer[i]=high[i]; wins++;
            } else
               {
                ExtLossBuffer[i]=high[i]; losses++;
               }
         }

      //--- results calculated
     }
//--- calculating total trades and ratio
   double total=wins+losses;
   double ratio=NormalizeDouble((wins/total)*100,2);
//--- print results
   Comment(wins+":"+losses+" ["+total+"] "+ratio+"% ");
//--- done
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Calculate Standard Deviation                                     |
//+------------------------------------------------------------------+
double StdDev_Func(int position,const double &close[],const double &MAprice[],int period)
  {
//--- variables
   double StdDev_dTmp=0.0;
//--- check for position
   if(position<period) return(StdDev_dTmp);
//--- calcualte StdDev
   for(int i=0;i<period;i++) StdDev_dTmp+=MathPow(close[position-i]-MAprice[position],2);
   StdDev_dTmp=MathSqrt(StdDev_dTmp/period);
//--- return calculated value
   return(StdDev_dTmp);
  }
//+------------------------------------------------------------------+

